#region Copyright
/*
 * Copyright 2005-2009 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
#endregion

using System.Collections.Generic;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using AddInCommon.Util;
using EnvDTE;
using EnvDTE80;
using Microsoft.VisualStudio.CommandBars;
using VSArrange.Config;
using StatusBar = EnvDTE.StatusBar;
using VSArrange.Filter;
using System.Text;
using Thread=System.Threading.Thread;

namespace VSArrange.Control
{
    /// <summary>
    /// \[VAvWFNgvf
    /// </summary>
    public class ArrangeControl
    {
        private const string REFRESH_BUTTON_NAME_SOLUTION = "SvWFNgvf̐";
        private const string REFRESH_BUTTON_NAME_PROJECT = "vWFNgvf̐";

        private readonly DTE2 _applicationObject;

        /// <summary>
        /// vWFNgڂƂȂt@C𔻕ʂ鐳K\
        /// </summary>
        private ItemAttachmentFilter _filterFile;

        /// <summary>
        /// t@Co^OtB^[
        /// </summary>
        public ItemAttachmentFilter FileterFile
        {
            set { _filterFile = value; }
            get { return _filterFile; }
        }

        /// <summary>
        /// vWFNgڂƂȂtH_𔻕ʂ鐳K\
        /// </summary>
        private ItemAttachmentFilter _filterFolder;

        /// <summary>
        /// tH_[o^OtB^[
        /// </summary>
        public ItemAttachmentFilter FilterFolder
        {
            set { _filterFolder = value; }
            get { return _filterFolder; }
        }
        
        /// <summary>
        /// RXgN^
        /// </summary>
        /// <param name="applicationObject"></param>
        public ArrangeControl(DTE2 applicationObject)
        {
            _applicationObject = applicationObject;
        }

        /// <summary>
        /// \[VENbNj[ɍڂǉĕԂ
        /// </summary>
        /// <param name="commandBar"></param>
        /// <returns></returns>
        public virtual CommandBarControl CreateSolutionContextMenuItem(CommandBar commandBar)
        {
            CommandBarButton refreshSolutuinButton =
                CommandBarUtils.CreateCommandBarControl<CommandBarButton>(commandBar);
            refreshSolutuinButton.Caption = REFRESH_BUTTON_NAME_SOLUTION;
            refreshSolutuinButton.Click += refreshSolutuinButton_Click;
            return refreshSolutuinButton;
        }

        /// <summary>
        /// vWFNgENbNj[ɍڂǉĕԂ
        /// </summary>
        /// <param name="commandBar"></param>
        /// <returns></returns>
        public virtual CommandBarControl CreateProjectContextMenuItem(CommandBar commandBar)
        {
            CommandBarButton refreshProjectButton =
                CommandBarUtils.CreateCommandBarControl<CommandBarButton>(commandBar);
            refreshProjectButton.Caption = REFRESH_BUTTON_NAME_PROJECT;
            refreshProjectButton.Click += refreshProjectButton_Click;
            return refreshProjectButton;
        }

        #region Cxg

        /// <summary>
        /// \[V{^NbNCxg
        /// </summary>
        /// <param name="Ctrl"></param>
        /// <param name="CancelDefault"></param>
        private void refreshSolutuinButton_Click(CommandBarButton Ctrl, ref bool CancelDefault)
        {
            Solution solution = _applicationObject.Solution;

            //  vWFNgǉtB^̍XV
            //  ݒ肪ύXꂽ_ŗ\ߔ񓯊œǂłǂ
            //  ptH[}XIɐOɓǂł肪ȂƎv邽
            //  Pɂ{RȂ߂ŌĂяo
            RefreshConfigInfo();
            try
            {
                foreach (Project project in solution.Projects)
                {
                    ArrangeProject(project);
                }
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message + "\n" + ex.StackTrace);
            }
            finally
            {
                StatusBarUtils.Clear(_applicationObject);
            }
        }

        /// <summary>
        /// vWFNg{^NbNCxg
        /// </summary>
        /// <param name="Ctrl"></param>
        /// <param name="CancelDefault"></param>
        private void refreshProjectButton_Click(CommandBarButton Ctrl, ref bool CancelDefault)
        {
            IDictionary<string, Project> refreshedProjects = new Dictionary<string, Project>();
            SelectedItems items = _applicationObject.SelectedItems;

            //  vWFNgǉtB^̍XV
            //  ݒ肪ύXꂽ_ŗ\ߔ񓯊œǂłǂ
            //  ptH[}XIɐOɓǂł肪ȂƎv邽
            //  Pɂ{RȂ߂ŌĂяo
            RefreshConfigInfo();
            try
            {
                foreach (SelectedItem selectedItem in items)
                {
                    Project currentProject = selectedItem.Project;
                    if (refreshedProjects.ContainsKey(currentProject.FullName))
                    {
                        //  XVς̃vWFNg͖
                        continue;
                    }

                    ArrangeProject(currentProject);
                    refreshedProjects[currentProject.FullName] = currentProject;
                }
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message + "\n" + ex.StackTrace);
            }
            finally
            {
                refreshedProjects.Clear();
                StatusBarUtils.Clear(_applicationObject);
            }
        }

        #endregion

        #region tbV

        /// <summary>
        /// vWFNg̃tbV
        /// </summary>
        /// <param name="project"></param>
        protected virtual void ArrangeProject(Project project)
        {
            if (string.IsNullOrEmpty(project.FullName))
            {
                //  vWFNgĂȂvf͖
                return;    
            }

            string projectDirPath = Path.GetDirectoryName(project.FullName);
            ProjectItems projectItems = project.ProjectItems;

            string statusLabel = string.Format("vWFNg[{0}]̗vf𐮗Ă܂B", project.Name);
            ArrangeDirectories(projectDirPath, projectItems);

            _applicationObject.StatusBar.Text = string.Format("{0}̐I܂B", project.Name);
        }

        /// <summary>
        /// fBNg̐
        /// </summary>
        /// <param name="dirPath"></param>
        /// <param name="projectItems"></param>
        protected virtual void ArrangeDirectories(string dirPath, ProjectItems projectItems)
        {
            _applicationObject.StatusBar.Text = string.Format("{0}𐮗Ă܂B", dirPath);
            IDictionary<string, ProjectItem> fileItems = new Dictionary<string, ProjectItem>();
            IDictionary<string, ProjectItem> folderItems = new Dictionary<string, ProjectItem>();
            IList<ProjectItem> deleteTarget = new List<ProjectItem>();

            string basePath = dirPath + Path.DirectorySeparatorChar;
            //  t@CAtH_A폜ΏۂɐU蕪
            foreach (ProjectItem projectItem in projectItems)
            {
                string currentPath = basePath + projectItem.Name;
                if(Directory.Exists(currentPath))
                {
                    if(_filterFolder.IsPassFilter(projectItem.Name))
                    {
                        //  ۂɑ݂ĂĊvWFNgɂo^
                        if(!folderItems.ContainsKey(currentPath))
                        {
                            folderItems.Add(currentPath, projectItem);
                        }
                    }
                    else
                    {
                        deleteTarget.Add(projectItem);
                    }
                }
                else if(File.Exists(currentPath))
                {
                    if(_filterFile.IsPassFilter(projectItem.Name))
                    {
                        if(!fileItems.ContainsKey(currentPath))
                        {
                            fileItems.Add(currentPath, projectItem);
                        }
                    }
                    else
                    {
                        deleteTarget.Add(projectItem);
                    }
                }
                else
                {
                    deleteTarget.Add(projectItem);
                }
            }

            ////  fBNgǉ
            //  XgōsĂċNɂ֌Ŵ
            //  fBNgǉ͐VKXbhł̎s͍sȂ
            DirectoryAppender directoryAppender = new DirectoryAppender(
                dirPath, _filterFolder, projectItems, folderItems);
            directoryAppender.Execute();

            ////  t@Cǉ
            FileAppender fileAppender = new FileAppender(
                dirPath, _filterFile, projectItems, fileItems);
            System.Threading.Thread fileThread = new Thread(
                new ThreadStart(fileAppender.Execute));
            fileThread.Start();

            ////  svȗvf͍폜
            ProjectItemRemover projectItemRemover = new ProjectItemRemover(deleteTarget);
            System.Threading.Thread removeThread = new Thread(
                new ThreadStart(projectItemRemover.Execute));
            removeThread.Start();

            //  ctH_ɑ΂ēl̏ċAIɎs
            foreach (string projectDirPath in folderItems.Keys)
            {
                ArrangeDirectories(projectDirPath, folderItems[projectDirPath].ProjectItems);
            }

        }

        #endregion

        /// <summary>
        /// ݒēǂݍ
        /// </summary>
        private void RefreshConfigInfo()
        {
            //  ݒǂݍ
            ConfigInfo configInfo = ConfigFileManager.ReadConfig(PathUtils.GetConfigPath());
            if (configInfo != null)
            {
                //  vWFNgvfǉOtB^[ݒ
                if (configInfo.FilterFileStringList != null)
                {
                    _filterFile.Clear();
                    _filterFile.AddFilters(configInfo.FilterFileStringList);
                }

                if (configInfo.FilterFolderStringList != null)
                {
                    _filterFolder.Clear();
                    _filterFolder.AddFilters(configInfo.FilterFolderStringList);
                }
            }
        }

        #region NX

        /// <summary>
        /// fBNgǉNX
        /// </summary>
        private class DirectoryAppender
        {
            private readonly string _dirPath;
            private readonly ItemAttachmentFilter _filter;
            private readonly ProjectItems _projectItems;
            private readonly IDictionary<string, ProjectItem> _folderItems;

            public DirectoryAppender(
                string dirPath, ItemAttachmentFilter filter,
                ProjectItems projectItems, IDictionary<string, ProjectItem> folderItems)
            {
                _dirPath = dirPath;
                _filter = filter;
                _projectItems = projectItems;
                _folderItems = folderItems;
            }

            /// <summary>
            /// fBNgǉs
            /// </summary>
            public void Execute()
            {
                string[] subDirPaths = Directory.GetDirectories(_dirPath);
                foreach (string subDirPath in subDirPaths)
                {
                    string[] dirPathParts = subDirPath.Split('\\');
                    string dirName = dirPathParts[dirPathParts.Length - 1];
                    if (_filter.IsPassFilter(dirName) &&
                        !_folderItems.ContainsKey(subDirPath))
                    {
                        //  ܂ǉĂȂ̂̂ݒǉ
                        ProjectItem newItem = _projectItems.AddFromDirectory(subDirPath);
                        _folderItems.Add(subDirPath, newItem);
                    }
                }
            }
        }

        /// <summary>
        /// t@CǉNX
        /// </summary>
        private class FileAppender
        {
            private readonly string _dirPath;
            private readonly ItemAttachmentFilter _filter;
            private readonly ProjectItems _projectItems;
            private readonly IDictionary<string, ProjectItem> _fileItems;

            public FileAppender(
                string dirPath, ItemAttachmentFilter filter,
                ProjectItems projectItems, IDictionary<string, ProjectItem> fileItems)
            {
                _dirPath = dirPath;
                _filter = filter;
                _projectItems = projectItems;
                _fileItems = fileItems;
            }

            /// <summary>
            /// t@Cǉs
            /// </summary>
            public void Execute()
            {
                string[] subFilePaths = Directory.GetFiles(_dirPath);
                foreach (string subFilePath in subFilePaths)
                {
                    string[] filePathParts = subFilePath.Split('\\');
                    string fileName = filePathParts[filePathParts.Length - 1];
                    if (_filter.IsPassFilter(fileName) &&
                        !_fileItems.ContainsKey(subFilePath))
                    {
                        //  ܂ǉĂȂ̂̂ݒǉ
                        _projectItems.AddFromFile(subFilePath);
                    }
                }
            }
        }

        /// <summary>
        /// vWFNgvf폜NX
        /// </summary>
        private class ProjectItemRemover
        {
            private readonly IList<ProjectItem> _deleteTarget;

            public ProjectItemRemover(IList<ProjectItem> deleteTarget)
            {
                _deleteTarget = deleteTarget;
            }

            public void Execute()
            {
                foreach (ProjectItem projectItem in _deleteTarget)
                {
                    projectItem.Remove();
                }
            }
        }

        #endregion
    }
}